(* Wolfram Language Package *)

BeginPackage[
	"QuantumAlgebra`Agreements`", 
	{ 
		"QuantumAlgebra`"
	}
]

(* Exported symbols added here with SymbolName::usage *)  

(* ::Section:: *)
(* Exported Symbols *)


QAPureSelectDefault::usage = 
	"The pure version of set selection"

QAReset::usage =
	"If a re-run of initialisation is desired, one can invoke this function first " <>
	"It enables a clean state. Call it AFTER QAPureSelectDefault."	

QAPureGetIndexCharacter::usage = 
	"The pure version of get Index Character"

QAPureSetIndexCharacter::usage =
	"The pure version of set Index Character"

QAResetReplacementRules::usage =
	"Resets a given set of Replacement Rules"

QAAddReplacementRules::usage =
	"Adds to a given set of Replacement Rules"

QAGetReplacementRules::usage =
	"Gets a given set of Replacement Rules"

QAResetExplicitIntegrationVariables::usage = 
	"Resets a given set of Explicit Integration Variables"

QAAddExplicitIntegrationVariables::usage = 
	"Adds to a given set of Explicit Integration Variables"

QAGetExplicitIntegrationVariables::usage = 
	"Gets a given set of Explicit Integration Variables"

QAResetAssumptions::usage = 
	"Resets a given set of Assumptions"

QAAddAssumptions::usage = 
	"Adds to a given set of Assumptions"

QAGetAssumptions::usage = 
	"Gets a given set of Assumptions"

QASetTimeConstraint::usage =
	"Defines the Time Constraint for QASimplify (default: 300)"

QAGetTimeConstraint::usage =
	"Gets the Time Constraint for QASimplify (default: 300)"

ResetNotScriptedSymbols::usage = 
	"Resets a set of Symbols not to be used as scripted symbols."

AddNotScriptedSymbols::usage = 
	"Defines a set of Symbols not to be used as scripted symbols."

GetNotScriptedSymbols::usage = 
	"Gets a set of Symbols not to be used as scripted symbols."


(* ::Section:: *)
(* Implementation of the Package *)

Begin["`Private`"] (* Begin Private Context *) 

		

(* ::Subsection:: *)
(* Public Rules *)

(* The pure version of set selection *)
QAPureSelectDefault[st_ : "default"] := ( set[default, st]; QAReset[] )

(* If a re-run of initialisation is desired, one can invoke this function first.
   It enables a clean state. Call it AFTER QAPureSelectDefault.
*)
QAReset[] :=
	Module[{},
		resetnb[notScriptedSymbols];
		resetnb[replacementRules];
		resetnb[variables];
		resetnb[timeConstraint];
		resetnb[assumptions];
		resetnb[indexCharacter];
	]

(* The pure version of get Index Character *)
QAPureGetIndexCharacter[set_ : Null, def: {_String, _List} : {"Mixed", {}}] := get[indexCharacter, set, def]

(* The pure version of set Index Character *)
QAPureSetIndexCharacter[characterdef: {character_String, vars_List}, st_ : Null] := set[indexCharacter, characterdef, st]

(* Resets the given set of Replacement Rules *)
QAResetReplacementRules[set_ : Null] := reset[replacementRules, set]

(* Adds to the given set of Replacement Rules *)
QAAddReplacementRules[rules_List, set_ : Null] := add[replacementRules, rules // Flatten, set]

(* Gets the given set of Replacement Rules
   Replacement Rules can be predefined -> that's why the extra handling 
*)
QAGetReplacementRules[set_ : Null] := 
	Module[{},
		If[MatchQ[set, _String] && KeyExistsQ[replacementRules, set],
			Lookup[replacementRules, set],
			get[replacementRules, set]
		]
	]

(* Resets the given set of Integration Variables *)
QAResetExplicitIntegrationVariables[set_ : Null] := reset[variables, set]

(* Adds to the given set of Integration Variables *)
QAAddExplicitIntegrationVariables[vars_List, set_ : Null] := add[variables, vars, set]

(* Gets the given set of Integration Variables *)
QAGetExplicitIntegrationVariables[set_ : Null] := get[variables, set]

(* Reset the assumptions of a given set *)
QAResetAssumptions[set_ : Null] := reset[assumptions, set]

(* Adds to the assumptions of a given set *)
QAAddAssumptions[ass_List, set_ : Null] := add[assumptions, ass, set]

(* Gets the assumptions of a given set *)
QAGetAssumptions[set_ : Null] := get[assumptions, set]

(* Sets the Time Constraint of a given set *)
QASetTimeConstraint[constraint_, st_ : Null] := set[timeConstraint, constraint, st]

(* Gets the Time Constraint of a given set *)
QAGetTimeConstraint[set_ : Null, def_ : 300.] := get[timeConstraint, set, def]

(* Resets the given set of Not Scripted Symbols *)
ResetNotScriptedSymbols[set_ : Null] := reset[notScriptedSymbols, set]

(* Adds to the given set of Not Scripted Symbols *)
AddNotScriptedSymbols[symbols_List, set_ : Null] := add[notScriptedSymbols, symbols, set]

(* Gets the given set of Not Scripted Symbols *)
GetNotScriptedSymbols[set_ : Null] := get[notScriptedSymbols, set]


(* ::Subsection:: *)
(* Internal Rules and Data *)

(* Gets the given set(entry) of the given association, also given a default values *)
SetAttributes[get, HoldFirst]
get[assoc_?AssociationQ, set_ : Null, def_ : {}] := Lookup[assoc, getSet[set], def]

(* Sets the given set(entry) of the given association to given value *)
SetAttributes[set, HoldFirst]
set[assoc_?AssociationQ, val_, st_ : Null] := AssociateTo[assoc, getSet[st] -> val]

(* Resets the given set(entry) of the given association to empty list *)
SetAttributes[reset, HoldFirst]
reset[assoc_?AssociationQ, set_ : Null] := AssociateTo[assoc, getSet[set] -> {}]

(* Resets the sets(entries) of the given association, belonging to current notebook, to empty list *)
SetAttributes[resetnb, HoldFirst]
resetnb[assoc_?AssociationQ] := 
	Module[{default, keys},
		default = getSet[Null];
		keys = Keys[KeySelect[assoc, StringStartsQ[#, default]&]];
		KeyDropFrom[assoc, keys]
	]

(* Adds to the given set(entry) of the given association the given list *)
SetAttributes[add, HoldFirst]
add[assoc_?AssociationQ, lst_List, set_ : Null] := 
	Module[{old, new},
		old = Lookup[assoc, getSet[set], {}];
		new = Join[old, lst];
		AssociateTo[assoc, getSet[set] -> new]
	]

(* Returns the set name *)
getSet[set_String] :=
	Module[{default},
		default = getSet[Null];
		If[MatchQ[default, _String], 
			default ~~ ":" ~~ set, 
			set
		]
	] 

getSet[Null] := 
	Module[{nb, set},
		nb = notebookName[];
		set = Lookup[default, nb, nb];
		set
	]


(*	The Notebook Name - to be used as Association key
	Name requires unnecessary CPU resources -> use Notebook as key 
*)
notebookName[] := EvaluationNotebook[]


(* ::Subsection:: *)
(* Variables *)

(* Named sets of Symbols which are not scripted *)
notScriptedSymbols = <| |>

(* Named sets of Replacement Rules *)
replacementRules = <| |>

(* Named sets of Explicit Integration / Summation Variables *)
variables = <| |>

(* Named Time Constraints for Simplify *)
timeConstraint = <| |>

(* Named sets of Assumptions for Simplify *)
assumptions = <| |>

(* Named Index Characters, grouped with related variables *)
indexCharacter = <| |>						(* Maybe Contravariant, Covariant, Mixed *)

(* The notebook's default sets *)
default = <| |>


(* End Private Context *)
End[]

EndPackage[]