(* Wolfram Language package *)
(*
	This package establishes calculations with Differential Operators.
	It uses the QAD function for further rules.
*)

BeginPackage[
    "QuantumAlgebra`DifferentialOperators`", 
    { 
        "QuantumAlgebra`", "QuantumAlgebra`RulesSupplement`", "QuantumAlgebra`Options`", "QuantumAlgebra`Analyze`" 
    }]



Begin["`Private`"]

(* COMMENT: Experience shows that Infix Notations of CircleTimes and CircleDot do work in modules
*)


(* We try to establish an operator form of QAD together with the operations Plus, CircleTimes (= operator application) and CircleDot (= operator composition) *)
(* \[CircleTimes] = Operator = Apply *)
(* \[CircleDot] = Operator = Compose *)

(* Right associativity of Apply *)
CircleTimes[a_, b_, c_] :=
	a \[CircleTimes] (b \[CircleTimes] c)

(* Compose and Apply combined 
   Apply can be used from left to right (normal case) or from right to left depending on operand order
*)
QAD[seq__] \[CircleDot] fac_ \[CircleTimes] ex_ :=
	QAD[seq] \[CircleTimes] (fac \[CircleTimes] ex)

fac_ \[CircleDot] QAD[seq__] \[CircleTimes] ex_ :=
    fac \[CircleTimes] (QAD[seq] \[CircleTimes] ex)

ex_ \[CircleTimes] QAD[seq__] \[CircleDot] fac_ :=
    QAD[seq] \[CircleTimes] (fac \[CircleTimes] ex)

ex_ \[CircleTimes] fac_ \[CircleDot] QAD[seq__] :=
    fac \[CircleTimes] (QAD[seq] \[CircleTimes] ex)


(* Required! Tremendous simplification *)
0 \[CircleDot] QAD[seq__] := 0

QAD[seq__] \[CircleDot] 0 := 0


(* Times as part of an operator *)
Times[c_, op_] \[CircleTimes] ex_ /; FreeQ[c, _QAD] && !FreeQ[op, _QAD] :=
    c (op \[CircleTimes] ex)


(* Main definition of Apply *)
QAD[seq__] \[CircleTimes] ex_ :=
    QAD[ex, seq]

ex_ \[CircleTimes] QAD[seq__] :=
    QAD[ex, seq]


(* Matrix operation with Apply *)
ex1_List \[CircleTimes] ex2_List :=
    Inner[CircleTimes, ex1, ex2, Plus]

ex1_List \[SmallCircle] ex2_List :=
    Outer[CircleTimes, ex1, ex2]

ex1_List \[CircleTimes] ex2_ /; FreeQ[ex2, _QAD] :=
    Map[# \[CircleTimes] ex2&, ex1]

ex1_QAD \[CircleTimes] ex2_List :=
    Map[ex1 \[CircleTimes] #&, ex2]

(* Fallback of Apply without derivative *)
ex1_ \[CircleTimes] ex2_ /; FreeQ[ex1, _QAD] && FreeQ[ex2, _QAD] :=
    ex1 * ex2


(* Matrix operation with Compose *)
ex1_List \[CircleDot] ex2_List :=
    Inner[CircleDot, ex1, ex2, Plus]

ex1_List \[CircleDot] ex2_QAD :=
    Map[# \[CircleDot] ex2&, ex1]

ex1_QAD \[CircleDot] ex2_List :=
    Map[ex1 \[CircleDot] #&, ex2]

(* Fallback of Compose without derivative *)
ex1_ \[CircleDot] ex2_ /; FreeQ[ex1, _QAD] && FreeQ[ex2, _QAD] :=
    ex1 * ex2

(* Apply a Sum of Operators *)
Unprotect[Plus];
(smd1_ + smd2:__) \[CircleTimes] ex_ /; !FreeQ[smd1, _QAD] :=
    smd1 \[CircleTimes] ex + (Plus@@{ smd2 }) \[CircleTimes] ex
Protect[Plus];




End[]

EndPackage[]
