(* Mathematica package *)

BeginPackage["QuantumAlgebra`Options`"]



QAOptions::usage =
	"QAOptions[Verbosity -> no] enables maximum print level no of Print messages from Quantum Algebra, " <>
	"QAOptions[Verbosity -> 0] disables them. Initial state is disabled."

If[!ValueQ[MaxLines],
	MaxLines::usage = "The MaxLines option: Infinity by default."
]

If[!ValueQ[PrintStack],
	PrintStack::usage = "The PrintStack option: None, Full, Recursive."
]

If[!ValueQ[StackPattern],
	StackPattern::usage = "The StackPattern option: None by default."
]

If[!ValueQ[Verbosity],
	Verbosity::usage = "The Verbosity option: 0 by default. Defines the Print Levels to print out."
]

If[!ValueQ[DefinitionLevel],
	DefinitionLevel::usage = "The DefinitionLevel option: 3 by default."
]

If[!ValueQ[Recursive],
	Recursive::usage = "A value for the PrintStack option."
]

If[!ValueQ[PrtLevel],
	PrtLevel::usage = "The Level to associate with a Print, 1 per default."
]

If[!ValueQ[Indent],
	Indent::usage = "The Indentation wo be used for Print, 1 per default."
]

QAPrint::usage =
	"A print function controlled by options."

QATiming::usage =
	"Measures the calculation of an expression and signs it with a time stamp."

QASniff::usage = 
	"Prints out an expression at a certain stage of an evaluation chain and returns the expression unchained."


Begin["`Private`"]

(* ::Section:: *)
(* Region Print and Logging Support *)

Options[QAOptions] = 
{
	Verbosity -> 0,
	MaxLines -> Infinity,
	PrintStack -> None,
	StackPattern -> None,
	LinesAlreadyPrinted -> 0,
	
	DefinitionLevel -> 3
}

Options[QAPrint] = 
{
	Indent -> 1,
	PrtLevel -> 1
}


(* Used to pre-set the Options for QAPrint or other purposes *)
QAOptions[OptionsPattern[]] :=
	SetOptions[
		QAOptions, 
		{ 
			Verbosity -> OptionValue[Verbosity], 
			MaxLines -> OptionValue[MaxLines],
			PrintStack -> OptionValue[PrintStack], 
			StackPattern -> OptionValue[StackPattern],
			LinesAlreadyPrinted -> OptionValue[LinesAlreadyPrinted],

			DefinitionLevel -> OptionValue[DefinitionLevel] 
		}]


(* Prints diagnostic expressions depending on set Options *)
SetAttributes[QAPrint, { HoldAll }]

QAPrint[expr___, OptionsPattern[]] := 
	Module[
		{
			prtlvl = OptionValue[QAPrint, PrtLevel],
			indent = StringJoin @@ Table["  ", OptionValue[QAPrint, Indent] - 1],
			linesprinted = OptionValue[QAOptions, LinesAlreadyPrinted],
			
			maxlines = OptionValue[QAOptions, MaxLines],
			ps = OptionValue[QAOptions, PrintStack],
			sp = OptionValue[QAOptions, StackPattern],
			verbosity = OptionValue[QAOptions, Verbosity],
			tr
		},
		tr = 
			Which[
				ps =!= None && sp === None, 
				Stack[], 
				ps =!= None, 
				Stack[sp]
			];
		If[linesprinted < maxlines && prtlvl <= verbosity, 
			Print[indent, expr];
			linesprinted++;
			Which[
				ps === Full,
				linesprinted++; 
				Print[indent, "Stack: ", tr],
				ps === Recursive && Count[tr, Print] > 0,
				linesprinted++;
				Print[indent, "Recursive Call: ", tr]
			];
			SetOptions[QAOptions, { LinesAlreadyPrinted -> linesprinted }];		 
		];
	]

(* Measures the calculation of an expression and signs it with a time stamp *)	
SetAttributes[QATiming, {HoldAll, SequenceHold}]

QATiming[ex_] := 
	Module[{date, elapsed, res},
		date = DateString[];
		{ elapsed, res } = Timing[ex];
		{ { date, elapsed }, res }
	]

(* Prints out an expression at a certain stage of an evaluation chain and returns the expression unchained *)
QASniff[ex_, descr_String : "Sniffing..."] :=
	Module[{},
		QAPrint[descr, PrtLevel->2];
		QAPrint[ex, PrtLevel->2];
		ex
	]

(*****************************************************************************)

End[]

EndPackage[]
